/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#if MIXCAST_LWRP || MIXCAST_URP
using BlueprintReality.MixCast.Data;
using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Rendering;
#if MIXCAST_LWRP
using UnityEngine.Rendering.LWRP;
#elif MIXCAST_URP
using UnityEngine.Rendering.Universal;
#endif

namespace BlueprintReality.MixCast
{
    public class MixCastRendererFeature : ScriptableRendererFeature
    {
        private class CameraPasses
        {
            public MixCastScriptableRenderPasses.SaveAlphaPass GrabBgAlpha { get; private set; }
            public MixCastScriptableRenderPasses.ApplyCutoffPass Cutoff { get; private set; }
            public MixCastScriptableRenderPasses.SaveAlphaPass GrabFgAlpha { get; private set; }

            public CameraPasses(ExpCameraBehaviour cam)
            {
                GrabBgAlpha = new MixCastScriptableRenderPasses.SaveAlphaPass(cam, ExpCameraBehaviour.RenderMode.FullRender);
                Cutoff = new MixCastScriptableRenderPasses.ApplyCutoffPass(cam);
                GrabFgAlpha = new MixCastScriptableRenderPasses.SaveAlphaPass(cam, ExpCameraBehaviour.RenderMode.Foreground);
            }
        }

        private static MixCastRendererFeature instance;
        private static MixCastRendererFeature Instance
        {
            get
            {
                if (instance == null)
                    instance = ScriptableObject.CreateInstance<MixCastRendererFeature>();
                return instance;
            }
        }

        private static List<ForwardRendererData> registeredWithRenderers = new List<ForwardRendererData>();
        public static void RegisterRenderer(ForwardRendererData renderer)
        {
            if (!registeredWithRenderers.Contains(renderer))
                renderer.rendererFeatures.Add(Instance);
            registeredWithRenderers.Add(renderer);
        }
        public static void UnregisterRenderer(ForwardRendererData renderer)
        {
            registeredWithRenderers.RemoveAt(registeredWithRenderers.LastIndexOf(renderer));
            if (!registeredWithRenderers.Contains(renderer))
                renderer.rendererFeatures.Remove(Instance);
            if (registeredWithRenderers.Count == 0)
                Destroy(Instance);
        }

        public static event Action<Camera> BeforeCameraRender;

        private Dictionary<ExpCameraBehaviour, CameraPasses> passes = new Dictionary<ExpCameraBehaviour, CameraPasses>();

        public override void Create()
        {

        }
        public override void AddRenderPasses(ScriptableRenderer renderer, ref RenderingData renderingData)
        {
            BeforeCameraRender?.Invoke(renderingData.cameraData.camera);

            if (ExpCameraBehaviour.CurrentlyRendering == null)
                return;

            if (!passes.TryGetValue(ExpCameraBehaviour.CurrentlyRendering, out CameraPasses camPasses))
                passes.Add(ExpCameraBehaviour.CurrentlyRendering, camPasses = new CameraPasses(ExpCameraBehaviour.CurrentlyRendering));

            switch(ExpCameraBehaviour.CurrentlyRendering.CurrentRenderMode)
            {
                case ExpCameraBehaviour.RenderMode.FullRender:
                    if (MixCastSdkData.ProjectSettings.grabUnfilteredAlpha && ExpCameraBehaviour.CurrentlyRendering.FullRenderSetAsTransparent)
                    {
                        camPasses.GrabBgAlpha.Prepare(renderer.cameraColorTarget);
                        renderer.EnqueuePass(camPasses.GrabBgAlpha);
                    }
                    break;

                case ExpCameraBehaviour.RenderMode.Foreground:
                    renderer.EnqueuePass(camPasses.Cutoff);
                    if (MixCastSdkData.ProjectSettings.grabUnfilteredAlpha)
                    {
                        camPasses.GrabFgAlpha.Prepare(renderer.cameraColorTarget);
                        renderer.EnqueuePass(camPasses.GrabFgAlpha);
                    }
                    break;
            }
        }
    }
}
#endif
