/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

using System.Collections.Generic;
using UnityEngine;

namespace BlueprintReality.MixCast.Data
{
    public static class MixCastSdkData
    {
        private const string PROJECT_SETTINGS_FILENAME = "MixCast_ProjectSettings";

        //Build-time
        public static MixCastProjectSettings ProjectSettings { get; private set; }

        //Runtime 
        public static Shared.ExperienceMetadata ExperienceInfo { get; set; }

        public static Shared.Desktop Desktop { get; set; }

        public static List<Shared.Viewfinder> Viewfinders { get; set; }
        public static Shared.Viewfinder GetViewfinderWithId(string id)
        {
            for (int i = 0; i < Viewfinders.Count; i++)
                if (Viewfinders[i].Identifier == id)
                    return Viewfinders[i];
            return null;
        }

        public static List<Shared.VirtualCamera> Cameras { get; set; }
        public static Shared.VirtualCamera GetCameraWithId(string id)
        {
            for (int i = 0; i < Cameras.Count; i++)
                if (Cameras[i].Identifier == id)
                    return Cameras[i];
            return null;
        }

        public static List<Shared.VideoInput> VideoInputs { get; set; }
        public static Shared.VideoInput GetVideoInputWithId(string id)
        {
            for (int i = 0; i < VideoInputs.Count; i++)
                if (VideoInputs[i].Identifier == id)
                    return VideoInputs[i];
            return null;
        }

        static MixCastSdkData()
        {
            ExperienceInfo = new Shared.ExperienceMetadata();

            Cameras = new List<Shared.VirtualCamera>();
            VideoInputs = new List<Shared.VideoInput>();
            Viewfinders = new List<Shared.Viewfinder>();
            Desktop = new Shared.Desktop()
            {
                MaxDisplayingCameras = 1,
                DisplayingCameraIds = new List<string>(),
                UiZoomFactor = 1
            };

            ProjectSettings = Resources.Load<MixCastProjectSettings>(PROJECT_SETTINGS_FILENAME);
        }
    }
}
