/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#if UNITY_STANDALONE_WIN
using BlueprintReality.Thrift.SharedTextures;
using System;
using UnityEngine;

namespace BlueprintReality.SharedTextures
{
    [Serializable]
    public class SharedTextureReceiver : IDisposable
    {
        public static Func<string, SharedTextureReceiver> Create { get; private set; }
        static SharedTextureReceiver()
        {
            Create = (string texId) =>
            {
                return new SharedTextureReceiver(texId);
            };
        }
        public static void OverrideCreateFunc(Func<string, SharedTextureReceiver> newFunc)
        {
            Create = newFunc;
        }

        public string TextureId { get; protected set; }

        protected Texture texture;
        public Texture Texture { get { return texture; } }

        protected SharedTex textureInfo;

        protected SharedTextureReceiver(string texId)
        {
            TextureId = texId;
            SharedTextureSending.OnTextureSent += RefreshTextureFromEvent;
            RefreshTextureInfo();
        }
        public virtual void Dispose()
        {
            if (Texture != null)
                SharedTextureReceiving.TryRelease(TextureId, ref texture);
            SharedTextureSending.OnTextureSent -= RefreshTextureFromEvent;
        }

        public virtual void RefreshTextureInfo()
        {
            if (SharedTextureSending.TryGetLocalTexture(TextureId, ref texture))
            {
                textureInfo = null;
                return;
            }
            SharedTextureReceiving.TryRequest(TextureId, ref texture, ref textureInfo);
        }
        public void RefreshTextureObject()
        {
            if (textureInfo == null)
                return;
            SharedTextureReceiving.RespondToTextureUpdate(TextureId, textureInfo, ref texture);
        }

        //Useful for streamlining event driven updates
        public void RefreshTextureFromEvent(string evTextureId, SharedTex evTextureInfo)
        {
            if (TextureId == evTextureId)
            {
                if(textureInfo == null)
                {
                    texture = null;
                    if (SharedTextureSending.TryGetLocalTexture(TextureId, ref texture))
                        return;
                }

                if (SharedTextureSending.TryGetLocalTexture(TextureId, ref texture))
                {
                    textureInfo = null;
                    return;
                }

                textureInfo = evTextureInfo;
                RefreshTextureObject();
            }
        }
    }
}
#endif
